actors = {}
players = {}


local function update_normal(f, dt)
end


function draw_normal(f)
    if f.sx and not f.sy then
        --love.graphics.draw(f.img, f.x, f.y, 0, -1, 1)
        love.graphics.draw(IMG_SPRITES, f.img, f.x, f.y, 0, -1, 1, 20, 0)
    elseif f.sx and f.sy then
        love.graphics.draw(IMG_SPRITES, f.img, f.x, f.y, 0, -1, -1, 20, 20)
    elseif not f.sx and f.sy then
        love.graphics.draw(IMG_SPRITES, f.img, f.x, f.y, 0, 1, -1, 0, 20)
    elseif not f.sx and not f.sy then
        love.graphics.draw(IMG_SPRITES, f.img, f.x, f.y, 0, 1, 1, 0, 0)
    end
end


local function draw_scaled(f)
    love.graphics.setColor(1, 1, 1, f.c)
    love.graphics.draw(IMG_SPRITES, f.img, f.x-f.s*10+10, f.y-f.s*10+10, 0, f.s, f.s, 0, 0)
    love.graphics.setColor(1, 1, 1, 1)
end


function make_actor(x, y, img)
    local f = {}
    f.id = 0
    f.x=x or 0
    f.y=y or 0
    f.img=img
--    if img ~= nil then
--        f.w=img:getWidth()
--        f.h=img:getHeight()
        f.draw=draw_normal
--    end
    f.update=update_normal
    f.refresh=update_normal
    f.sx,f.sy=false,false
    table.insert(actors, f)
    return f
end


function make_spike(x,y,img)
    local f=make_actor(x, y, img)
    --local f=make_actor(x, y, QUAD_LAVA)
    for k,v in pairs(TILES_TBL) do
        if v == img then
            f.id = k
        end
    end
    return f
end


function make_stone(x,y)
    local f=make_actor(x, y, QUAD_STONE)
    f.id = ID_STONE
    return f
end


function make_key(x,y)
    local f=make_actor(x, y, TBL_KEYS[1])
    f.id = ID_KEY
    f.t = 0
    f.i = 1
    f.update = function(f, dt)
        f.t = f.t + dt
        if f.t > .2 then
            f.t = 0
            f.i = f.i + 1
            if f.i > #TBL_KEYS then
                f.i = 1
            end
            f.img = TBL_KEYS[f.i]
        end
        if is_touching_player(f.x, f.y) then
            SND_KEY:play()
            level[f.y/20+1][f.x/20+1] = ID_EMPTY
            for i=#actors,1,-1 do
                if actors[i] == f then
                    table.remove(actors, i)
                    for j=1,#actors do
                        actors[j].refresh(actors[j])
                    end
                    break
                end
            end
        end
    end
    return f
end


function make_lava(x,y)
    local f=make_actor(x, y, QUAD_LAVA)
    f.id = ID_LAVA
    return f
end


function make_empty(x,y)
    local f=make_actor(x, y, QUAD_EMPTY)
    f.id = ID_EMPTY
    return f
end


function make_jumper(x,y)
    local f=make_actor(x, y, QUAD_JUMPER)
    f.id = ID_JUMPER
    return f
end


function show_fps(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("FPS: "..love.timer.getFPS(), f.x, f.y)
    end
    return f
end


function show_dt(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("dt: "..love.timer.getDelta(), f.x, f.y)
    end
    return f
end


function show_pdx(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("DX: "..player.dx, f.x, f.y)
    end
    return f
end


function show_pdy(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("DY: "..player.dy, f.x, f.y)
    end
    return f
end


function show_px(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("X: "..player.x, f.x, f.y)
    end
    return f
end


function show_py(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("Y: "..player.y, f.x, f.y)
    end
    return f
end


function show_stats(x, y)
    local f = make_actor(x,y)
    f.i = 0
    f.draw=function(f)
        for k, v in pairs(love.graphics.getStats()) do
            love.graphics.print(k..": "..v, f.x, f.y+(f.i*20))
            f.i = f.i + 1
        end
        f.i = 0
    end
    return f
end


function show_countactors(x, y)
    local f = make_actor(x,y)
    f.draw=function(f)
        love.graphics.print("Actors: "..#actors, f.x, f.y)
    end
    return f
end


function timed_action(ms, actions, args)
    local f=make_actor()
    f.draw=function(f)end
    f.goalms = ms/1000
    f.args = args
    f.ms = 0
    f.update=function(f, dt)
        -- increment timer
        f.ms = f.ms + dt
        -- if timer reaches goal? Then Action!
        if f.ms >= f.goalms then
            if f.args == nil then
                actions()
            else
                actions(f.args)
            end
            -- remove myself from actors:
            for i, v in ipairs(actors) do
                if v == f then
                    table.remove(actors, i)
                    break
                end
            end
        end
    end
    return f
end


function make_winnersprite(x, y)
    local f=make_actor(x, y, QUAD_WINNER)
    timed_action(125, function()
        f.y = f.y - 2
        timed_action(125, function()
            f.y = f.y - 2
            timed_action(125, function()
                f.y = f.y - 2
                timed_action(125, function()
                    f.y = f.y + 2
                    timed_action(125, function()
                        f.y = f.y + 2
                        timed_action(125, function()
                            f.y = f.y + 2
                            timed_action(250, winner.finish)
                        end)
                    end)
                end)
            end)
        end)
    end)
    return f
end


function make_clickanim(x, y, spr)
    SND_PLACE:play()
    local f=make_actor(x, y, spr)
    f.s = 2
    f.c = .2
    --f._s = 2
    --f._c = .8
    f.img = spr
    f.draw = draw_scaled
    f.t = 1
    --[[
    timed_action(40, function()f.s,f.c=1.8,.8 end)
    timed_action(80, function()f.s,f.c=1.6,.6 end)
    timed_action(120, function()f.s,f.c=1.4,.4 end)
    timed_action(160, function()f.s,f.c=1.2,.2 end)
    timed_action(200, function()
        f.s,f.c=1,0
    end)
    --]]
    f.update = function(f, dt)
        f.s = f.s - 4*dt
        f.c = f.c + 3*dt
    end
    timed_action(240, function()
        for i=#actors,1,-1 do
            if actors[i] == f then
                table.remove(actors, i)
                break
            end
        end
    end)
    return f
end


function make_cursor()
    local f=make_actor(0, 0)
    f.img = PLAYERCURSOR[PLAYERCURPOS]
    f.check_pos = false
    f.can_place = false
    f.disable_time = 0
    f.update = update_cursor
    f.draw = draw_cursor
    return f
end


function draw_cursor(f)
    if f.can_place then
        love.graphics.setColor(.6, .6, 1, .8)
    else
        love.graphics.setColor(.6, 0, 0, .8)
    end
    draw_normal(f)
    love.graphics.draw(IMG_SPRITES, QUAD_CURSOR, math.floor((f.x+9)/20)*20, math.floor((f.y+9)/20)*20)
    love.graphics.setColor(1, 1, 1, 1)
end


function update_cursor(f)
    -- Update from Mousepos
    f.x = MOUSE.x-9
    f.y = MOUSE.y-9
    local v = level[math.floor((f.y+9)/20)+1][math.floor((f.x+9)/20)+1]
    if v == 0
    or v == ID_EMPTY
    or v == ID_PLAYER
    or v == ID_PLAYER_RIGHT
    or v == ID_PLAYER_LEFT
    or v == ID_PLAYER_NOTJUMP
    or v == ID_PLAYER_REVERT
    or v == ID_RED_UD
    or v == ID_RED_LR
      then
        if is_touching_others(f) then
            f.can_place = false
        else
            f.can_place = true
        end
    else
        f.can_place = false
    end

    if MOUSE.leftclick and not f.can_place then
        SND_NOPLACE:play()
    end
    -- Place Element if Player wants it
    if MOUSE.leftclick and f.can_place then
        --for i, v in ipairs(TILES_TBL) do
        for k, v in pairs(SPAWNTILES_TBL) do
            --print(k..": "..f.id)
            if v == f.img then
                level[flr((MOUSE.y)/20)][flr((MOUSE.x)/20)] = k
                local ix,iy = math.floor((f.x+9)/20)*20, math.floor((f.y+9)/20)*20
                spawn(k, ix, iy)
                -- Swap Position with last actor to avoid Graphic-Overlapping with Cursor.
                for i, v in ipairs(actors) do
                    if v == f then
                        actors[i],actors[#actors]=actors[#actors],actors[i]
                        make_whud()
                        break
                    end
                end
                --actors[1],actors[#actors] = actors[#actors],actors[1]
                make_clickanim(ix, iy, SPAWNTILES_TBL[k])
                -- Toggle States off, because Cursor should be turned "Off" as long as the clickanim works.
                f.draw = function(f)end
                f.update = function(f)end
                timed_action(240, function(f)f.draw,f.update = draw_cursor,update_cursor end, f)
                break
            end
        end
        -- Remove from List
        table.remove(PLAYERCURSOR, PLAYERCURPOS)
        f.check_pos = true
    -- Check changing Block
    elseif MOUSE.wheel_up then
        PLAYERCURPOS = PLAYERCURPOS + 1
        f.check_pos = true
    -- Check changing Block
    elseif MOUSE.wheel_down then
        PLAYERCURPOS = PLAYERCURPOS - 1
        f.check_pos = true
    end
    if f.check_pos then
        -- every move gone? Then delete myself...
        if #PLAYERCURSOR == 0 then
            for i=#actors,1,-1 do
                if actors[i] == f then
                    table.remove(actors, i)
                    break
                end
            end
        elseif PLAYERCURPOS < 1 then
            PLAYERCURPOS = #PLAYERCURSOR
        elseif PLAYERCURPOS > #PLAYERCURSOR then
            PLAYERCURPOS = 1
        end
        f.check_pos = false
        f.img = PLAYERCURSOR[PLAYERCURPOS]
    end
end


function make_door(x, y)
    local f=make_actor(x, y, QUAD_DOOR)
    f.id = ID_DOOR
    f.i = 1
    f.t = 0
    f.refresh = function(f)
        local keys = 0
        local y,x,s
        for y=1,#level-1 do
            for x=1,#level[y] do
                s = level[y][x]
                if s == ID_KEY then
                    keys = keys + 1
                end
            end
        end
        -- no Keys? Then transform myself into a Flag.
        if keys == 0 then
            f.img = QUAD_EMPTY
            level[f.y/20+1][f.x/20+1] = ID_FLAG
            make_flag(f.x, f.y)
            for i=#actors,1,-1 do
                if actors[i] == f then
                    f.img = QUAD_EMPTY
                    f.draw(f)
                    table.remove(actors, i)
                    --break
                end
            end
        else
            if keys > 9 then keys = 9 end
            f.img = TBL_DOORS[keys]
        end
    end
    return f
end


function make_redenemy(x, y, is_leftright)
    local f=make_actor(x, y, QUAD_RED)
    if is_leftright then
        f.id = ID_RED_LR
        f._dx = 100
        f._dy = 0
    else
        f.id = ID_RED_UD
        f._dx = 0
        f._dy = 100
    end
    f.update = function(f, dt)
        -- Movement
        f.dx = f._dx*dt
        f.dy = f._dy*dt
        if is_solid(f.x+f.dx,f.y+f.dy) then
            f._dx = -f._dx
            f._dy = -f._dy
        else
            f.x = f.x + f.dx
            f.y = f.y + f.dy
        end

        -- Check Gameover
        if is_touching_player(f.x, f.y) then
            make_gameover()
        end
    end
    return f
end
