
local configfile = {}


configfile.settings = {
    vsync = true,
    fullscreen = false,
    highdpi = false,
    x = 640,
    y = 360,
    lang = "english.txt",
--[[
    k_up = "up",
    k_down = "down",
    k_left = "left",
    k_right = "right",
    k_jump = "z",
--]]
    borderless = true,
}


local FILENAME = "settings.cfg"


function configfile.get_lang()
    local ret = {}
    local line
    for line in love.filesystem.lines("langs/"..configfile.settings["lang"], "file") do
        ret[#ret+1] = line:gsub("\\n", "\n")
    end
    return ret
end


function configfile.write()
    -- vsync
    love.filesystem.write(FILENAME, tostring(configfile.settings["vsync"]))
    love.filesystem.append(FILENAME, "\n")
    -- fullscreen
    love.filesystem.append(FILENAME, tostring(configfile.settings["fullscreen"]))
    love.filesystem.append(FILENAME, "\n")
    -- highdpi
    love.filesystem.append(FILENAME, tostring(configfile.settings["highdpi"]))
    love.filesystem.append(FILENAME, "\n")
    -- res x
    love.filesystem.append(FILENAME, tostring(configfile.settings["x"]))
    love.filesystem.append(FILENAME, "\n")
    -- res y
    love.filesystem.append(FILENAME, tostring(configfile.settings["y"]))
    love.filesystem.append(FILENAME, "\n")
    -- Language
    love.filesystem.append(FILENAME, configfile.settings["lang"])
    love.filesystem.append(FILENAME, "\n")
    -- Keys
    love.filesystem.append(FILENAME, KEY.jump.scancode)
    love.filesystem.append(FILENAME, "\n")
    love.filesystem.append(FILENAME, KEY.down.scancode)
    love.filesystem.append(FILENAME, "\n")
    love.filesystem.append(FILENAME, KEY.left.scancode)
    love.filesystem.append(FILENAME, "\n")
    love.filesystem.append(FILENAME, KEY.right.scancode)
    love.filesystem.append(FILENAME, "\n")
    love.filesystem.append(FILENAME, KEY.accept.scancode)
    love.filesystem.append(FILENAME, "\n")
    love.filesystem.append(FILENAME, KEY.restart.scancode)
    love.filesystem.append(FILENAME, "\n")
    -- Borderless
    love.filesystem.append(FILENAME, tostring(configfile.settings["borderless"]))
    love.filesystem.append(FILENAME, "\n")

end


function configfile.read()
    local info
    info = love.filesystem.getInfo(FILENAME, "file")
    if info == nil then
        configfile.write()
    else
        local i = 0
        local line
        for line in love.filesystem.lines(FILENAME) do
            i = i + 1
            -- vsync
            if i==1 and line == "true" then configfile.settings["vsync"] = true
            elseif i==1 and line == "false" then configfile.settings["vsync"] = false
            -- fullscreen
            elseif i==2 and line == "true" then configfile.settings["fullscreen"] = true
            elseif i==2 and line == "false" then configfile.settings["fullscreen"] = false
            -- highdpi
            elseif i==3 and line == "true" then configfile.settings["highdpi"] = true
            elseif i==3 and line == "false" then configfile.settings["highdpi"] = false
            -- res x
            elseif i==4 then configfile.settings["x"] = tonumber(line)
            -- res y
            elseif i==5 then configfile.settings["y"] = tonumber(line)
            -- Language
            elseif i==6 then configfile.settings["lang"] = line
            -- Keys
            elseif i==7 then KEY.jump.scancode = line
            elseif i==8 then KEY.down.scancode = line
            elseif i==9 then KEY.left.scancode = line
            elseif i==10 then KEY.right.scancode = line
            elseif i==11 then KEY.accept.scancode = line
            elseif i==12 then KEY.restart.scancode = line
            -- borderless
            elseif i==13 and line == "true" then configfile.settings["borderless"] = true
            elseif i==13 and line == "false" then configfile.settings["borderless"] = false
            end
        end
    end
end


function configfile.get(key)
    return tostring(configfile.settings[key])
end


love.filesystem.setIdentity("Place the Tiles")
configfile.read()
return configfile
