love.graphics.setDefaultFilter("nearest", "linear", 0)
--[globals]

-- canvas
canvas = nil  -- for scaling
bga = nil  -- for escapemenu-screenshot
PLAYERLEVEL = 0  -- actual level
PLAYERCURSOR = {}  -- levelcursors
PLAYERCURPOS = 1  -- levelcursorposition


-- Keys - configfile.read() can overwrite scancode.
KEY = {}
KEY.jump = {scancode="up"}
KEY.left = {scancode="left"}
KEY.down = {scancode="down"}
KEY.right = {scancode="right"}
KEY.accept = {scancode="return"}
KEY.esc = {scancode="escape"} -- not configurable at the moment
KEY.restart = {scancode="r"}


for k,_ in pairs(KEY) do
    KEY[k].pressed = false
end

-- Key-changing-Mode on?
KEYMODE = false

-- Mouse
MOUSE = {}
MOUSE.x = 0
MOUSE.y = 0
MOUSE.leftclick = false
MOUSE.wheel_up = false
MOUSE.wheel_down = false
MOUSE.moved = false


-- Imports
game = require("game")
require("player")
require("actorlib")
require("data_snd")
require("data_img")
require("hud")
menuengine = require("menuengine")
configfile = require("configfile")
instructionmenu = require("instructionmenu")
instructionmenu2 = require("instructionmenu2")
mainmenu = require("mainmenu")
options = require("options")
quit = require("quit")
credits = require("credits")
controlmenu = require("controlmenu")
new_keymenu = require("new_keymenu")
err_keymenu = require("err_keymenu")
resmenu = require("resmenu")
episodemenu = require("episodemenu")
escapemenu = require("escapemenu")
winner = require("winner")
episodewinner = require("episodewinner")
local aspect_ratio = require('lib/aspect_ratio')


-- Mode
MODE_UPD = nil
MODE_DRAW = nil

-- Language-File
LANG = configfile.get_lang()

-- new Frame (for collision detection)
IS_NEWFRAME = true

--[/globals]


-- FPS-Accumulator
local FPS = 1 / 60
local FPS_BUFFER = 0


-- Internal res
local RES_X = 640
local RES_Y = 360




function love.load(args)
    canvas = love.graphics.newCanvas()
    set_window()
    --love.graphics.setDefaultFilter("nearest", "nearest")
    --love.graphics.setDefaultFilter("linear", "nearest")
    --love.graphics.setBlendMode("alpha", "premultiplied")

    menuengine.settings.sndMove = love.audio.newSource("sounds/pick.wav", "static")
    menuengine.settings.sndSuccess = love.audio.newSource("sounds/accept.wav", "static")

    MUS_MAINMENU:play()
    mainmenu.init()
end


function set_window()
    love.graphics.setCanvas()
    love.window.setMode(
        configfile.settings.x,
        configfile.settings.y, {
            vsync = configfile.settings.vsync,
            fullscreen = configfile.settings.fullscreen,
            fullscreentype = (configfile.settings.borderless and "desktop" or "exclusive"),
            highdpi = configfile.settings.highdpi,
            resizable = true,
        }
    )
    love.resize(configfile.settings.x, configfile.settings.y)
end

function love.update(dt)
    -- Prevent too high dt in some circumstances (for example, resizing or moving the Window).
    if dt > 0.1 then
        return
    else
        canvas:renderTo(function()
        love.graphics.clear()
        MODE_DRAW()
        end)
        MODE_UPD(dt)

        -- code to remove Inputs
        if not KEYMODE then
            for k, v in pairs(KEY) do
                if v.scancode ~= nil and not love.keyboard.isScancodeDown(v.scancode) then
                    KEY[k].pressed = false
                end
            end
            MOUSE.leftclick = false
            MOUSE.wheel_up = false
            MOUSE.wheel_down = false
            MOUSE.moved = false
            KEY.restart.pressed = false  -- Prevent Restart-Loop (High CPU-Usage) by Holding that Key.
        end
    end
end


function love.draw()
    --love.graphics.scale(love.graphics.getWidth()/RES_X,love.graphics.getHeight()/RES_Y)
    --love.graphics.clear(0, 0, 0, 255)
    --love.graphics.clear()
    if bga ~= nil then
        --love.graphics.push()
        love.graphics.setColor(.5, .5, .5)
        love.graphics.draw(bga)
        love.graphics.setColor(1, 1, 1)
        --love.graphics.pop()
    end
    love.graphics.draw(canvas, aspect_ratio.x, aspect_ratio.y, 0, aspect_ratio.scale)
end


function love.mousemoved(x, y, dx, dy, istouch)
--    if not configfile.settings.fullscreen and not configfile.settings.borderless then
--        x = x * (RES_X / love.graphics.getWidth())
--        y = y * (RES_Y / love.graphics.getHeight())
--        x = x * (RES_X / aspect_ratio.game_w)
--        y = y * (RES_Y / aspect_ratio.game_h)
--    end
    x, y = aspect_ratio:window_to_game_position(x, y)
    MOUSE.x = x
    MOUSE.y = y
    MOUSE.moved = true
    _check_mouse()
    menuengine.mousemoved(x, y)
end

function love.mousepressed(x, y, button, istouch, presses)
    if button == 1 then
        MOUSE.leftclick = true
        MOUSE.moved = true
    end
    _check_mouse()
end

-- Prevent "overflooding" Mouse-Cursor
function _check_mouse()
    if MOUSE.x < 0 then MOUSE.x = 0
    elseif MOUSE.y < 0 then MOUSE.y = 0 end
    if MOUSE.x > 640 then MOUSE.x = 640
    elseif MOUSE.y > 360 then MOUSE.y = 360 end
end

function love.keypressed(key, scancode, isrepeat)
    -- Keymode if User change Controls
    if KEYMODE then
        if scancode ~= "escape" then  -- forbid Rebinding Escape-Key
            new_keymenu.key(scancode)
            KEYMODE = false
        end
    else
        for k, v in pairs(KEY) do
            if KEY[k].scancode == scancode then
                KEY[k].pressed = true
            end
        end
    end
end

function love.resize(w, h)
    aspect_ratio:init(w, h, RES_X, RES_Y)
    canvas = love.graphics.newCanvas(aspect_ratio.dig_w, aspect_ratio.dig_h)
    bga = nil  -- Prevent "wrong-sized" Background in Escapemenu.
end


function love.wheelmoved(x, y)
    if y > 0 then
        MOUSE.wheel_up = true
    elseif y < 0 then
        MOUSE.wheel_down = true
    end
end
