

local options = {}
local menu
local TTF = love.graphics.newFont("fonts/inconsolata-bold.ttf", 20)


function options.finish(entry)
    -- Save & Go back to Mainmenu
    local c = menu.cursor
    if entry == "back" then
        configfile.write()
        mainmenu.init()
    elseif entry == "resolution" then
        resmenu.init()
    elseif entry == "fullscreen" then
        configfile.settings.fullscreen = not configfile.settings.fullscreen
        set_window()
        options.init()
        menu.cursor = c
    elseif entry == "vsync" then
        configfile.settings.vsync = not configfile.settings.vsync
        set_window()
        options.init()
        menu.cursor = c
    elseif entry == "highdpi" then
        configfile.settings.highdpi = not configfile.settings.highdpi
        set_window()
        options.init()
        menu.cursor = c
    elseif entry == "change_keyboard" then
        controlmenu.init()
    elseif entry == "borderless" then
        configfile.settings.borderless = not configfile.settings.borderless
        if configfile.settings.fullscreen then
            set_window()
        end
        options.init()
        menu.cursor = c
    elseif entry == "language" then
        -- TODO: Better change, not hardcoded please^^.
        if configfile.settings.lang == "deutsch.txt" then
            configfile.settings.lang = "english.txt"
        else
            configfile.settings.lang = "deutsch.txt"
        end
        LANG = configfile.get_lang()
        options.init()
        menu.cursor = c
    end
end


function options.update()
    menu:update()
    if KEY.esc.pressed then
        menu.cursor = 1
        menu:moveCursor(-1)
        menu:_finish()
        KEY.esc.pressed = false
    end
end


function options.draw()
    love.graphics.draw(IMG_BACKGROUND)
    love.graphics.setFont(TTF_TITLES)
    love.graphics.print({{1,.4,.4},LANG[46]}, 0, 20)
    love.graphics.setFont(TTF)
    menu:draw()
end


function options.init()
    MODE_UPD, MODE_DRAW = options.update, options.draw
    love.graphics.setFont(TTF)
    menu = menuengine.new(190, 100)
    local tmp
    -- Resolution
    tmp = menu:addEntry(LANG[5]..": "..tostring(configfile.settings.x).."x"..tostring(configfile.settings.y), options.finish, "resolution")
    if configfile.settings.fullscreen and configfile.settings.borderless then
        tmp.colorNormal = {.4,.4,.4}
    end
    -- Fullscreen
    menu:addEntry(LANG[7]..": "..(configfile.settings.fullscreen and LANG[10] or LANG[11]), options.finish, "fullscreen")
    -- Borderless
    tmp = menu:addEntry(LANG[27]..": "..(configfile.settings.borderless and LANG[10] or LANG[11]), options.finish, "borderless")
    tmp.x = tmp.x + 20
    if not configfile.settings.fullscreen then
        tmp.colorNormal = {.4,.4,.4}
    end
    -- Vsync
    menu:addEntry(LANG[6]..": "..(configfile.settings.vsync and LANG[10] or LANG[11]), options.finish, "vsync")
    -- HighDPI
    menu:addEntry(LANG[14]..": "..(configfile.settings.highdpi and LANG[10] or LANG[11]), options.finish, "highdpi")
    menu:addSep()
    -- Change Keyboard Settings
    menu:addEntry(LANG[8], options.finish, "change_keyboard")
    -- Change Controller Settings (TODO?)
    --menu:addEntry(LANG[9], options.finish, "change_controller")
    menu:addSep()
    -- Change Language
    menu:addEntry(LANG[13], options.finish, "language")
    menu:addSep()
    -- Go Back
    menu:addEntry(LANG[12], options.finish, "back")
    KEY.esc.pressed = false
end


return options
